/******************************************************************************/
/*                                                                            */
/* Descrition:                                                                */
/*   Implementation of UNIX-routines                                          */
/*     opendir                                                                */
/*     readdir                                                                */
/*     closedir                                                               */
/* Author:                                                                    */
/*   hna                                                                      */
/* Created:                                                                   */
/*   16-Nov-1992                                                              */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <descrip.h>
#include <rmsdef.h>
#include <dirent.h>
#include <string.h>

#define endif
#define endfor
#define VERSIZE(e)	(sizeof e->vms_versions / sizeof e->vms_versions[0])

extern char	*malloc();
extern char	*strrchr();
extern char	*strcpy();

/******************************************************************************/
/*                                                                            */
/* Open directory                                                             */
/*                                                                            */
/******************************************************************************/

DIR *opendir (p_name)
char *p_name;
{
  DIR	*lv_dd;
  char  lv_wildcards[4] = "";
  int	lv_length = 0;
  char  *lv_ptr;

  if ((lv_dd = (DIR *)malloc(sizeof *lv_dd)) == NULL)
    {
      errno = ENOMEM;
      return NULL;
    };
  endif;
  lv_ptr = strchr (p_name, ']');
  if (lv_ptr == NULL)
    {
      lv_ptr = p_name;
    }
   else
    {
      lv_ptr++;
    };
  endif;
  if (strlen (lv_ptr) == 0)
    {
      strcpy (lv_wildcards, "*.*");
      lv_length = 3;
    }
   else
    {
      if (strchr(lv_ptr, '.') == NULL)
        {
          strcpy (lv_wildcards, ".*");
          lv_length = 2;
        }
      endif;
    }
  endif;
  lv_dd->pattern = malloc ((unsigned int) (strlen (p_name) + lv_length + 1));
  if (lv_dd->pattern == NULL)
    {
      free ((char *) lv_dd);
      errno = ENOMEM;
      return NULL;
    };
  endif;
  (void) sprintf (lv_dd->pattern, "%s%s", p_name, lv_wildcards);
/*
  printf ("Pattern: %s\n", lv_dd->pattern);
*/
  lv_dd->context = 0;
  lv_dd->vms_wantversions = 0;
  lv_dd->pat.dsc$a_pointer = lv_dd->pattern;
  lv_dd->pat.dsc$w_length = strlen (lv_dd->pattern);
  lv_dd->pat.dsc$b_dtype = DSC$K_DTYPE_T;
  lv_dd->pat.dsc$b_class = DSC$K_CLASS_S;
  return lv_dd;
}


void vmsreaddirversions (p_dd, p_flag)
DIR	*p_dd;
int	p_flag;
{
  p_dd->vms_wantversions = p_flag;
}

/******************************************************************************/
/*                                                                            */
/* CLOSE directory                                                             */
/*                                                                            */
/******************************************************************************/

void closedir (p_dd)
DIR	*p_dd;
{
  free (p_dd->pattern);
  free ((char *)p_dd);
}


static void collectversions (p_dd)
DIR	*p_dd;
{
  struct dsc$descriptor_s 	lv_pat;
  struct dsc$descriptor_s 	lv_res;
  struct dirent			*lv_e;
  char				*lv_p;
  char				lv_buff[sizeof p_dd->entry.d_name];
  int				lv_i;
  char				*lv_text;
  long				lv_context;

  lv_e = &p_dd->entry;

  lv_i = strlen (p_dd->pattern);
  /* we would only need the string upto ']' or '>' (see below) */

  lv_text = malloc ((unsigned int)(lv_i + strlen (lv_e->d_name)+2+1));
  if (lv_text == NULL)
    {
      return;
    }
  endif;
  (void) strcpy (lv_text, p_dd->pattern);

/*  (void) sprintf (&lv_text[lv_i - 3], "%s;*", lv_e->d_name); */

  if (lv_p = strchr (lv_text, ']'))
    {
      *++lv_p = '\0';
    }
   else
    {
      if (lv_p = strchr (lv_text, '>'))
        {
          *++lv_p = '\0';
        }
       else
        {
          lv_text[0] = '\0';
        }
      endif;
    }
  endif;

  (void) strcat (lv_text, lv_e->d_name);

  lv_pat.dsc$a_pointer = lv_text;
  lv_pat.dsc$w_length = strlen (lv_text);
  lv_pat.dsc$b_dtype = DSC$K_DTYPE_T;
  lv_pat.dsc$b_class = DSC$K_CLASS_S;

  lv_res.dsc$a_pointer = lv_buff;
  lv_res.dsc$w_length = sizeof lv_buff - 2;
  lv_res.dsc$b_dtype = DSC$K_DTYPE_T;
  lv_res.dsc$b_class = DSC$K_CLASS_S;

  for (lv_context = 0; lv_e->vms_verscount < VERSIZE(lv_e); lv_e->vms_verscount++)
    {
      if (lib$find_file (&lv_pat, &lv_res, &lv_context) == RMS$_NMF || lv_context == 0L)
        {
          break;
        }
      endif;
      lv_buff [sizeof lv_buff - 1] = '\0';
      if (lv_p = strchr (lv_buff, ';'))
        {
          lv_e->vms_versions [lv_e->vms_verscount] = atoi (lv_p + 1);
        }
       else
        {
          lv_e->vms_versions [lv_e->vms_verscount] = -1;
        }
      endif;
    }
  endfor;
  free (lv_text);
}


struct dirent * readdir (p_dd)
DIR	*p_dd;
{
  struct dsc$descriptor_s 	lv_res;
  char				*lv_p;
  char				lv_buff[sizeof p_dd->entry.d_name];
  int				lv_i;
  int				lv_status;

  lv_res.dsc$a_pointer = lv_buff;
  lv_res.dsc$w_length = sizeof lv_buff - 2;
  lv_res.dsc$b_dtype = DSC$K_DTYPE_T;
  lv_res.dsc$b_class = DSC$K_CLASS_S;

  lv_status = lib$find_file (&p_dd->pat, &lv_res, &p_dd->context);
  if (lv_status == RMS$_NMF || lv_status == RMS$_FNF || p_dd->context == 0L)
    {
      return NULL;
    }
  endif;
  lv_buff [sizeof lv_buff - 1] = '\0';
  for (lv_p = lv_buff; !isspace (*lv_p); lv_p++)
    {
    }
  endfor;
  *lv_p = '\0';
  if (lv_p = strchr (lv_buff, ']'))
    {
      (void) strcpy (p_dd->entry.d_name, lv_p + 1);
    }
   else
    {
      if (lv_p = strchr (lv_buff, '>'))
        {
          (void) strcpy (p_dd->entry.d_name, lv_p + 1);
        }
       else
        {
          (void) strcpy (p_dd->entry.d_name, lv_buff);
        }
      endif;
    }
  endif;
  if (lv_p = strchr (p_dd->entry.d_name, ';'))
    {
      *lv_p = '\0';
    }
  endif;
  p_dd->entry.vms_verscount = 0;
  if (p_dd->vms_wantversions)
    {
      collectversions (p_dd);
    }
  endif;
  return &p_dd->entry;
}

long telldir (p_dd)
DIR	*p_dd;
{
  return p_dd->context;
}

void seekdir (p_dd, p_pos)
DIR	*p_dd;
long	p_pos;
{
  p_dd->context = p_pos;
}
