/* <math.h>
 *
 *	Run-time library math functions.
 */
#ifndef _MATH_H
#define _MATH_H
# ifdef __cplusplus
extern "C" {
# endif


/*
 * jda 27.07.94 added  -- start --
 */


#include <values.h>

/* On some systems, HUGE ought to be MAXFLOAT or IEEE infinity */

#ifndef HUGE
#define HUGE    MAXDOUBLE
#endif

#ifndef M_E
#define M_E         2.7182818284590452354
#endif

#ifndef M_LOG2E
#define M_LOG2E     1.4426950408889634074
#endif

#ifndef M_LOG10E
#define M_LOG10E    0.43429448190325182765
#endif

#ifndef M_LN2
#define M_LN2       0.69314718055994530942
#endif

#ifndef M_LN10
#define M_LN10      2.30258509299404568402
#endif

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif

#ifndef M_PI_2
#define M_PI_2      1.57079632679489661923
#endif

#ifndef M_1_PI
#define M_1_PI      0.31830988618379067154
#endif

#ifndef M_PI_4
#define M_PI_4      0.78539816339744830962
#endif

#ifndef M_2_PI
#define M_2_PI      0.63661977236758134308
#endif

#ifndef M_2_SQRTPI
#define M_2_SQRTPI  1.12837916709551257390
#endif

#ifndef M_SQRT2
#define M_SQRT2     1.41421356237309504880
#endif

#ifndef M_SQRT1_2
#define M_SQRT1_2   0.70710678118654752440
#endif

#ifndef PI                      /* as in stroustrup */
#define PI  M_PI
#endif

#ifndef PI2
#define PI2  M_PI_2
#endif





/*
 * jda 27.07.94 added  -- end --
 */




#ifdef __GFLOAT__
# define HUGE_VAL (8.988465674311578540726371186585e+307)
#else
# define HUGE_VAL (1.70141183460469229e+38)
#endif

/* these are also defined in <errno.h> */
#define EDOM	33	/* Argument too large */
#define ERANGE	34	/* Result too large */

/* alternate representation of doubles requires different library routines,
   with names prefixed by "vaxc$g" */
#if    __GFLOAT__
# define ___gdecl(_func,_arglist) _func _arglist __asm__("_vaxc$g" #_func)
#elif  __MIXED_FLOAT__
# define ___gdecl(_func,_arglist) _func _arglist __asm__("_vaxc$d" #_func)
#else
# define ___gdecl(_func,_arglist) _func _arglist
#endif

/* these are routines in VAXCRTL listed by K&R2's ANSI <math.h> description */
double	___gdecl(acos,	(double));
double	___gdecl(asin,	(double));
double	___gdecl(atan,	(double));
double	___gdecl(atan2, (double,double));
double	___gdecl(ceil,	(double));
double	___gdecl(cos,	(double));
double	___gdecl(cosh,	(double));
double	___gdecl(exp,	(double));
double	___gdecl(fabs,	(double));
double	___gdecl(floor, (double));
double	___gdecl(fmod,	(double,double));
double	___gdecl(frexp, (double,int *));
double	___gdecl(ldexp, (double,int));
double	___gdecl(log,	(double));
double	___gdecl(log10, (double));
double	___gdecl(modf,	(double,double *));
double	___gdecl(pow,	(double,double));
double	___gdecl(sin,	(double));
double	___gdecl(sinh,	(double));
double	___gdecl(sqrt,	(double));
double	___gdecl(tan,	(double));
double	___gdecl(tanh,	(double));

/* these two are in VAX C's <math.h> (and VAXCRTL) */
#ifndef _CABS_T
#define _CABS_T
typedef struct CABS_T { double real, imag; } cabs_t;
#endif	/*_CABS_T*/
double	___gdecl(cabs,	(cabs_t));	/* sqrt(.real*.real + .imag*.imag) */
double	___gdecl(hypot, (double,double));	/* sqrt(d1*d1 + d2*d2) */

#undef ___gdecl		/*__GFLOAT__*/

# ifdef __cplusplus
}
# endif
#endif	/*_MATH_H*/
